#!/usr/bin/env python

import os
import sys
import andle.version
from os.path import expanduser

DATA_PATH = expanduser("~") + "/.andle"


def setpath(path):
    if not os.path.exists(path):
        print("not exist path")
        sys.exit(0)
    with open(DATA_PATH, "w+") as file_:
        file_.write(os.path.abspath(path))
    print("setsdk:" + getpath())


def getpath():
    if not os.path.exists(DATA_PATH):
        print("set sdk path first")
        sys.exit(0)
    file = open(DATA_PATH)
    return file.read().rstrip('\n')


def load(path=""):
    data = {}
    if path == "":
        path = getpath()
    # find build tool
    find_config(data, "build-tools", path)

    # find sdk version
    find_config(data, "platforms", path)
    data["platforms"] = data["platforms"].replace("android-", "")

    # find dependencies
    data["dependency"] = {}
    find_dependency(data["dependency"], "", path +
                    "/extras/android/m2repository")
    find_dependency(data["dependency"], "", path +
                    "/extras/google/m2repository")

    return data


def find_config(data, name, path):
    for f in os.listdir(path + "/" + name + "/"):
        if os.path.isdir(path + "/" + name + "/" + f):
            update_value(f, data, name)


def find_dependency(data, tag, path):
    if find_dependency_folder(data, tag, path):
        return
    list = tag[1:].split("/")
    if len(list) < 2:
        return
    version = list.pop(len(list) - 1)
    name = list.pop(len(list) - 1)
    package = ".".join(list)

    update_value(version, data, package + ":" + name)


def find_dependency_folder(data, tag, path):
    resolved = False
    if os.path.exists(path):
        for f in os.listdir(path):
            if (os.path.isdir(path + "/" + f)):
                resolved = True
                find_dependency(data, tag + "/" + f, path + "/" + f)
    return resolved


def update_value(var, obj, key):
    if key in obj:
        if andle.version.newer(obj[key], var) > 0:
            obj[key] = var
    else:
        obj[key] = var



#!/usr/bin/env python

import andle.android
import andle.sdk

__version__ = "2.0.0"


def update(path, dryrun, remote, gradle, interact):
    data = sdk.load()
    android.update(path, data, dryrun, remote, gradle, interact)


def setsdk(path):
    sdk.setpath(path)



#!/usr/bin/python

import andle.http
from xml.dom import minidom

JCENTER_URL = "https://jcenter.bintray.com/"
GOOGLE_URL = "https://maven.google.com/"


def load(name, url=JCENTER_URL):
    request = andle.http.request(
        url + name.replace(".", "/").replace(":", "/") + "/maven-metadata.xml")
    try:
        DOMTree = minidom.parse(request)

        collection = DOMTree.documentElement
        versioning = collection.getElementsByTagName("versioning")[0]
        latest = versioning.getElementsByTagName("release")[0]
        version = latest.childNodes[0].data
        return version
    except BaseException:
        if url == JCENTER_URL:
            return load(name, GOOGLE_URL)
        else:
            print("fail to get version : " + name)
            return None



#!/usr/bin/env python


def newer(version1, version2):

    parts1 = [x for x in version1.split('.')]
    parts2 = [x for x in version2.split('.')]

    # fill up the shorter version with zeros ...
    lendiff = len(parts1) - len(parts2)
    if lendiff > 0:
        parts2.extend([0] * lendiff)
    elif lendiff < 0:
        parts1.extend([0] * (-lendiff))

    return cmp(parts1, parts2)


def cmp(parts1, parts2):
    for i, p in enumerate(parts1):
        p = str(p)
        s = str(parts2[i])
        if p.isdigit() and s.isdigit():
            p = int(p)
            s = int(s)
        ret = (p < s) - (p > s)
        if ret:
            return ret
    return 0



#!/usr/bin/python

try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen


def request(url):
    try:
        return urlopen(url)
    except Exception:
        return None



#!/usr/bin/env python

import os
import fnmatch
import andle.remote
import andle.gradle
try:
    import __builtin__
    input = getattr(__builtin__, 'raw_input')
except (ImportError, AttributeError):
    pass

COMPILE_TAGS = ["compile", "Compile", "api", "implementation", "Implementation"]


def update(path, data, dryrun=False, remote=False, gradle=False, interact=False):
    global is_dryrun
    is_dryrun = dryrun
    global check_remote
    check_remote = remote
    global check_gradle
    check_gradle = gradle
    global is_interact
    is_interact = interact

    for file in filter(path, "build.gradle"):
        parse_dependency(file, data)

    if check_gradle:
        gradle_version = andle.gradle.load()
        for file in filter(path, "gradle-wrapper.properties"):
            parse_gradle(file, gradle_version)


def filter(path, name):
    result = []
    for root, dir, files in os.walk(path):
        for file in fnmatch.filter(files, name):
            result.append(root + "/" + file)
    return result


def parse_gradle(path, version):
    print("check " + path)
    with open(path) as f:
        io = f.readlines()

    global modify
    modify = False
    new_data = ""

    global line
    for line in io:
        if line.startswith("distributionUrl"):
            update_value("distributionUrl", line[16:].strip(),
                         "https\://services.gradle.org/distributions/gradle-" + version + "-all.zip")
        new_data += line

    # save back
    save(path, new_data)


def parse_dependency(path, data):
    print("check " + path)
    with open(path) as f:
        io = f.readlines()

    new_data = ""
    global modify
    modify = False

    global line
    for line in io:
        word = line.split()
        if word.__len__() >= 2:
            check_dependency(word, data)
            check_classpath(word, data)
        new_data += line

    # save back
    save(path, new_data)


def check_dependency(word, data):
    first = word[0]
    # find compileSdkVersion tag
    if check_compile_sdk_version(first, word, data):
        return True
    # find buildToolsVersion tag
    elif first == "buildToolsVersion":
        buildToolsVersion = word[1].replace("\"", "").replace("\'", "")
        update_value("buildToolsVersion",
                     buildToolsVersion, data["build-tools"])
        return True
    # find compile tag
    elif any(first.endswith(compile) for compile in COMPILE_TAGS):
        check_version(word, data["dependency"], check_remote)
        return True
    else:
        return False


def check_compile_sdk_version(first, word, data):
    if first == "compileSdkVersion":
        platforms = word[1]
        if data["platforms"] == "N":
            update_value("compileSdkVersion", platforms, "android-N")
        else:
            update_value("compileSdkVersion", platforms, data["platforms"])
        return True
    return False


def check_classpath(word, data):
    if check_gradle and word[0].startswith("classpath"):
        check_version(word, data["dependency"], check_gradle)
        return True
    else:
        return False


def check_version(word, deps, check_online):
    string = word[1]
    if string.startswith("'") or string.startswith("\""):
        dep = string.split(string[0])[1]
        tag = dep[:dep.rfind(":")]
        version = get_version(dep)

        if tag in deps:
            update_value(tag, version, deps[tag])
        elif check_online:
            online_version = andle.remote.load(tag)
            update_value(tag, version, online_version)
            deps[tag] = online_version


def get_version(dep):
    version = dep[dep.rfind(":") + 1:]

    if "@" in version:
        version = version[:version.find("@")]
    return version


def update_value(name, old, new):
    if not_new_version(old, new):
        return
    print(name + ": " + old + " -> " + new)

    if is_interact and input("> ") == "n":
        print("not modify")
        return
    global modify
    modify = True
    global line
    line = line.replace(old, new)


def not_new_version(old, new):
    if old == new or new == None:
        return True
    if andle.version.newer(new, old) > 0:
        return True
    return False


def save(path, new_data):
    if modify:
        if not is_dryrun:
            f = open(path, 'w')
            f.write(new_data)
            f.close()
            print("done")
    else:
        print("ok")



#!/usr/bin/python

import json
import andle.http
import codecs

URL = "http://services.gradle.org/versions/current"


def load(url=URL):
    try:
        reader = codecs.getreader("utf-8")
        data = json.load(reader(andle.http.request(url)))
        return data["version"]
    except Exception:
        print("fail to connect url: " + url)
        return None
